@@ -2,8 +2,8 @@
  * String Utility Library
  *
  * $RCSfile: libdgstr.c,v $
- * $Revision: 1.1 $
- * $Date: 2009/11/06 08:41:54 $
+ * $Revision: 1.1.1.1 $
+ * $Date: 2009/02/17 09:36:25 $
  */
 #include <stdio.h>
 #include <stdlib.h>
@@ -13,6 +13,10 @@
 
 #include "libdgstr.h"
 
+#ifdef HAVE_CONFIGH
+#include "../../config.h"
+#endif
+
 
 /*--- ؿ ---*/
 
@@ -324,8 +328,8 @@ dg_realloc(void *obuf, size_t len)
 char *
 str2code(char *from, char *to, char *inbuf)
 {
-    int     inleft;
-    int     outleft = 0;
+    size_t inleft;
+    size_t outleft = 0;
     char   *ip;
     char   *op;
     char   *buf = NULL;
