@@ -1,7 +1,7 @@
 /*
  * samma
  *
- * Copyright (C) 2006,2007,2008 DesigNET, INC.
+ * Copyright (C) 2006,2007,2008,2011 DesigNET, INC.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,8 +20,8 @@
 
 /*
  * $RCSfile: mailzip.c,v $
- * $Revision: 1.21 $
- * $Date: 2011/08/15 07:10:43 $
+ * $Revision: 1.22 $
+ * $Date: 2011/12/28 03:22:21 $
  */
 
 #include <stdio.h>
@@ -46,6 +46,7 @@
 #include "mailsave.h"
 #include "zipconv.h"
 #include "samma_policy.h"
+#include "samma_autobcc.h"
 #include "global.h"
 
 #define MLFIPRIV(ctx)        ((struct mlfiPriv *) smfi_getpriv(ctx))
@@ -76,6 +77,17 @@ mlfi_connect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *addr)
     /* set RcptTo check flag */
     priv->mlfi_rcptcheck = BEFOR_RCPTCHECK;
 
+    /* set encription flag */
+    priv->mlfi_encstatus = ENC;
+
+    /* set AutoBcc check flag */
+    if (priv->mlfi_conf->cf_autobccoption[0] == 'Y' || priv->mlfi_conf->cf_autobccoption[0] == 'y') {
+	priv->mlfi_bccoption = ENABLE_AUTOBCC;
+    }
+    else {
+	priv->mlfi_bccoption = DISABLE_AUTOBCC;
+    }
+
     /* create mailinfo structure */
     minfo = malloc(sizeof *minfo);
     if(minfo == NULL) {
@@ -159,10 +171,14 @@ mlfi_envfrom(SMFICTX *ctx, char **envfrom)
 
         return SMFIS_TEMPFAIL;
     } else if (ret == NOT_ENC) {
-        free(fromaddr);
-        mlfi_cleanup(ctx, MLFIABORT);
+	if (priv->mlfi_bccoption == DISABLE_AUTOBCC) {
+	    free(fromaddr);
+	    mlfi_cleanup(ctx, MLFIABORT);
 
-        return SMFIS_ACCEPT;
+	    return SMFIS_ACCEPT;
+	}
+	/* save envription status */
+	priv->mlfi_encstatus = NOT_ENC;
     }
     /* save from addr temporally */
     priv->mlfi_savefrom = fromaddr;
@@ -232,8 +248,21 @@ mlfi_header(SMFICTX *ctx, char *headerf, char *headerv)
 
     DEBUGLOG("mlfi_header start");
 
+    if (priv->mlfi_encstatus == NOT_ENC) {
+	return SMFIS_CONTINUE;
+    }
+
     /* RcptTo Check */
     if (priv->mlfi_rcptcheck == BEFOR_RCPTCHECK) {
+	if (priv->mlfi_bccoption == ENABLE_AUTOBCC) {
+	    ret = add_bccaddr(priv->mlfi_conf, priv->mlfi_savefrom,
+			     &priv->mlfi_savercpt, &priv->mlfi_savebcc);
+	    if (ret != SUCCESS) {
+		mlfi_cleanup(ctx, MLFIABORT);
+		return SMFIS_TEMPFAIL;
+	    }
+	}
+
 	ret = search_rcptaddr(priv->mlfi_conf, priv->mlfi_savefrom, 
 			      priv->mlfi_savercpt, &passlist, &rdmpasslist);
 	if (ret < 0) {
@@ -245,8 +274,14 @@ mlfi_header(SMFICTX *ctx, char *headerf, char *headerv)
 	if (ret == NOT_ENC) {
 	    free_rcptinfo(passlist);
 	    free_rcptinfo(rdmpasslist);
-	    mlfi_cleanup(ctx, MLFIABORT);
-	    return SMFIS_ACCEPT;
+	    if (priv->mlfi_bccoption == DISABLE_AUTOBCC) {
+		mlfi_cleanup(ctx, MLFIABORT);
+		return SMFIS_ACCEPT;
+	    }
+	    /* save envription status */
+	    priv->mlfi_encstatus = NOT_ENC;
+	    priv->mlfi_rcptcheck = AFTER_RCPTCHECK;
+	    return SMFIS_CONTINUE;
 	}
 
 	/* save encryption address list */
@@ -296,6 +331,10 @@ mlfi_eoh(SMFICTX *ctx)
         return SMFIS_TEMPFAIL;
     }
 
+    if (priv->mlfi_encstatus == NOT_ENC) {
+	return SMFIS_CONTINUE;
+    }
+
     /* write new line between header and body */
     if (mailsave_write(priv->mlfi_minfo, priv->mlfi_conf, "\n", 1) == -1 ) {
         /* write error */
@@ -323,6 +362,10 @@ mlfi_body(SMFICTX *ctx, u_char *bodyp, size_t bodylen)
 
     DEBUGLOG("mlfi_body start");
 
+    if (priv->mlfi_encstatus == NOT_ENC) {
+	return SMFIS_CONTINUE;
+    }
+
     /* write body */
     if (mailsave_write(priv->mlfi_minfo, priv->mlfi_conf, (char *)bodyp, bodylen) == -1 ) {
         /* write error */
@@ -341,7 +384,7 @@ mlfi_body(SMFICTX *ctx, u_char *bodyp, size_t bodylen)
 sfsistat
 mlfi_eom(SMFICTX *ctx)
 {
-    int ret;
+    int ret, i;
     struct mlfiPriv *priv;
     if ((priv = MLFIPRIV(ctx)) == NULL) {
 	log(ERR_GET_PRIV, "mlfi_eom");
@@ -351,6 +394,21 @@ mlfi_eom(SMFICTX *ctx)
 
     DEBUGLOG("mlfi_eom start");
 
+    if (priv->mlfi_savebcc != NULL) {
+	for (i = 0; (priv->mlfi_savebcc + i)->rcpt_addr != NULL; i++) {
+	    if (smfi_addrcpt(ctx, (priv->mlfi_savebcc + i)->rcpt_addr) != MI_SUCCESS) {
+		log(ERR_AUTOBCC, "mlfi_eom", "smfi_addrcpt() failed.");
+		mlfi_cleanup(ctx, MLFIABORT);
+		return SMFIS_TEMPFAIL;
+	    }
+	}
+    }
+
+    if (priv->mlfi_encstatus == NOT_ENC) {
+	mlfi_cleanup(ctx, MLFIABORT);
+	return SMFIS_ACCEPT;
+    }
+
     /* Encryption Process */
     ret = zip_convert_mail(ctx, priv->mlfi_minfo, priv->mlfi_conf, 
 			   priv->mlfi_savefrom, priv->mlfi_passlist, 
@@ -437,6 +495,11 @@ mlfi_cleanup(SMFICTX *ctx, int flag)
 	priv->mlfi_rdmpasslist = NULL;
     }
 
+    if (priv->mlfi_savebcc != NULL) {
+	free_rcptlist(priv->mlfi_savebcc);
+	priv->mlfi_savebcc = NULL;
+    }
+
     if (flag == MLFICLOSE) {
         /* config relase */
         if (priv->mlfi_conf != NULL) {
@@ -601,6 +664,7 @@ main(int argc, char *argv[])
     }
 
     /* set register */
+    smfilter.xxfi_flags |= SMFIF_ADDRCPT;
     if(smfi_register(smfilter) == MI_FAILURE) {
         fprintf(stderr, ERR_MILTER_REGISTER, strerror(errno));
 	fprintf(stderr, "\n");
