@@ -20,8 +20,8 @@
 
 /*
  * $RCSfile: zipconv.c,v $
- * $Revision: 1.21 $
- * $Date: 2011/07/29 03:03:03 $
+ * $Revision: 1.23 $
+ * $Date: 2013/01/09 08:18:13 $
  */
 
 #include <stdio.h>
@@ -168,13 +168,27 @@ zip_convert_mail(SMFICTX *ctx, struct mailinfo *minfo, struct config *cfg,
                 org_p = p->passwd;	/* save original password */
                 p->passwd = cfg->cf_fixedpassnotifypass;
 	        temp = tmpl_init();
+
+
 	        if (tmpl_tag_replace(temp, mz, p, from, &confirmation_message) != 0){
+                    p->passwd = org_p;	/* restore original password */
+	            tmpl_release(temp);
 	            mailzip_clean(mz);
 	            return ZIP_CONVERT_FAILED;
 	        }
                 p->passwd = org_p;	/* restore original password */
 	        tmpl_release(temp);
 
+                // set header field "References"
+                if (strcmp(cfg->cf_references, "yes") == 0) {
+                    ret = set_references(minfo->ii_mbuf, &confirmation_message);
+                    if (ret == ERROR) {
+                        free(confirmation_message);
+                        mailzip_clean(mz);
+                        return ZIP_CONVERT_FAILED;
+                    }
+                }
+
 	        if (passwd_sendmail(cfg, from, confirmation_message) != 0) {
 	            free(confirmation_message);
 	            mailzip_clean(mz);
@@ -221,11 +235,22 @@ zip_convert_mail(SMFICTX *ctx, struct mailinfo *minfo, struct config *cfg,
 
 	temp = tmpl_init();
 	if (tmpl_tag_replace(temp, mz, rdmpasslist, from, &notify_message) != 0){
+	    tmpl_release(temp);
 	    mailzip_clean(mz);
 	    return ZIP_CONVERT_FAILED;
 	}
 	tmpl_release(temp);
 
+        // set header field "references"
+        if (strcmp(cfg->cf_references, "yes") == 0) {
+            ret = set_references(minfo->ii_mbuf, &notify_message);
+            if (ret == ERROR) {
+                free(notify_message);
+                mailzip_clean(mz);
+                return ZIP_CONVERT_FAILED;
+            }
+        }
+
 	if (passwd_sendmail(cfg, from, notify_message) != 0) {
 	    free(notify_message);
 	    mailzip_clean(mz);
@@ -1003,3 +1028,100 @@ mk_encpath(struct config *cfg, struct mailzip *mz)
 
     return 0;
 }
+
+/* get_msgid
+ * 
+ * get the message id from file attached mail
+ *
+ * Args:
+ *    char **buf        buffer of set the message id
+ *    char *mail        mail data of file attached mail
+ *
+ * Return:
+ *    ERROR      -1      error
+ *    SUCCESS     0      success
+ */
+int get_msgid(char **buf, char *mail) {
+
+    //variables
+    int id_len = 0;
+    char *p = NULL;
+    char *head = NULL;
+    char *tail = NULL;      
+    char *tmp = NULL;
+
+    // search Message-ID field
+    p = strstr(mail, MSG_ID);   
+    if (p == NULL) {
+        return ERROR;
+    }
+
+    // search "<"
+    head = strchr(p, '<');
+    if (head == NULL) {
+        return ERROR;
+    }
+    // search ">"
+    tail = strchr(head, '>');
+    if (tail == NULL) {
+        return ERROR;
+    }
+    // check the length of Message-ID
+    id_len = strlen(head) - strlen(tail) + 1;
+
+    // allocate memory for Message-ID(2 is '\n' and '\0')
+    tmp = (char *)calloc(REF_LEN + id_len + 2, sizeof(char));
+    if (tmp == NULL) {
+        return ERROR;
+    }
+    *buf = tmp;
+
+    // set References field
+    strcat(*buf, REF);
+    memcpy(*buf + strlen(REF), head, id_len);
+    strcat(*buf, "\n\0");
+
+    return SUCCESS;
+}
+
+/* set_references
+ * 
+ * add header field "References" to password confirming mail
+ *
+ * Args:
+ *    char *mail_data	strings of mail data include header of file 
+ *                      attached mail
+ *    char *passwd_mail mail data of password confirming mail
+ *
+ * Return:
+ *    ERROR      -1      error
+ *    SUCCESS     0      success
+ */
+int set_references(char *mail_data, char **passwd_mail) {
+
+    // variables
+    char *tmp = NULL;
+    char *ref = NULL;
+    int ret = 0;
+
+    //get message id
+    ret = get_msgid(&ref, mail_data);
+    if (ret == ERROR) {
+        return ERROR;
+    }
+
+    // allocate memory for mail 
+    tmp = realloc(*passwd_mail, strlen(ref) + strlen(*passwd_mail) + 1);
+    if (tmp == NULL) {
+        free(ref);
+        return ERROR;
+    }
+    *passwd_mail = tmp;
+
+    // set references
+    memmove(*passwd_mail + strlen(ref), *passwd_mail, strlen(*passwd_mail) + 1);
+    memcpy(*passwd_mail, ref, strlen(ref));
+    free(ref);
+
+    return SUCCESS;
+}
