@@ -20,8 +20,8 @@
 
 /*
  * $RCSfile: zipconv.h,v $
- * $Revision: 1.12 $
- * $Date: 2011/07/29 03:03:03 $
+ * $Revision: 1.13 $
+ * $Date: 2013/01/09 08:18:13 $
  */
 #include <libmilter/mfapi.h>
 #include "mailzip_config.h"
@@ -60,6 +60,10 @@
 
 #define MESSAGE_FILENAME	"RFC822.eml"
 
+#define REF     "References: "
+#define MSG_ID  "Message-ID:"
+#define REF_LEN 12
+
 struct mailzip {
     const char *subject;
     char *date;
@@ -99,6 +103,8 @@ int mk_encpath(struct config *, struct mailzip *);
 int add_attach_file(struct mailzip *, struct rcptinfo *);
 int drop_file(GMimeObject *, struct config *, struct mailzip *);
 void free_rcptinfo(struct rcptinfo *);
+int get_msgid(char **, char *);
+int set_references(char *, char **);
 
 /* Return Values */
 #define PM_SUCCESS		0
